this.battle_formation_spec <- this.inherit("scripts/skills/skill", {
	m = {
		Stacks = 0,
		Bonus = 5
	},
	function create()
	{
		this.m.ID = "perk.battle_formation_spec";
		this.m.Name = "Battle Formation";
		this.m.Description = "Stand my ground! Being in formation gives bonus to melee and ranged skills per each allie with same kind of weapon.";
		this.m.Icon = "ui/perks/battle_formation.png";
		this.m.Type = this.Const.SkillType.Perk | this.Const.SkillType.StatusEffect;
		this.m.Order = this.Const.SkillOrder.Perk | this.Const.SkillType.StatusEffect;
		this.m.IsActive = false;
		this.m.IsStacking = false;
		this.m.IsHidden = false;
	}

	function getDescription()
	{
		if (this.getContainer().getActor().isPlacedOnMap())
		{
			if (this.m.Stacks == 0)
			{
				return "Has no allie near with same weapon to get a bonus."
			}

			return "Has " + this.m.Stacks + " allies with same weapon. Gains [color=" + this.Const.UI.Color.PositiveValue + "]" + this.m.Bonus * this.m.Stacks + "[/color] bonus to melee and ranged skill.";
		}
		
		return "Stand my ground! Being in formation gives [color=" + this.Const.UI.Color.PositiveValue + "]" + this.m.Bonus + "[/color] bonus to melee and ranged skills per each allie with same kind of weapon.";
	}

	function onUpdate(_properties)
	{
		local actor = this.getContainer().getActor();

		if (!actor.isPlacedOnMap())
		{
			return;
		}

		local myTile = actor.getTile();
		local item = this.getContainer().getActor().getItems().getItemAtSlot(this.Const.ItemSlot.Mainhand);
		this.m.Stacks = 0;

		if (item == null)
		{
			return;
		}

		local weaponType = item.m.Categories;
		for( local i = 0; i != 6; i = ++i )
		{
			if (!myTile.hasNextTile(i))
			{
			}
			else
			{
				local tile = myTile.getNextTile(i);

				if (!tile.IsEmpty && tile.IsOccupiedByActor && this.Math.abs(myTile.Level - tile.Level) <= 1)
				{
					local entity = tile.getEntity();

					if (actor.getFaction() == entity.getFaction())
					{
						item = entity.getItems().getItemAtSlot(this.Const.ItemSlot.Mainhand);
						if (item == null)
						{
							continue;
						}

						if (item.m.Categories == weaponType)
						{
							this.m.Stacks++;
						}
					}
				}
			}
		}
	}

	function onAfterUpdate( _properties )
	{
		if (this.m.Stacks != 0)
		{
			_properties.MeleeSkill += this.m.Bonus * this.m.Stacks;
			_properties.RangedSkill += this.m.Bonus * this.m.Stacks;
		}
	}

	function onCombatStarted()
	{
		this.m.Stacks = 0;
	}

	function onCombatFinished()
	{
		this.skill.onCombatFinished();
		this.m.Stacks = 0;
	}
});
